/*                 DEM19C - FUNCION DE ONDA TETAEDRICA
  SUPERFICIE (PLTSV) Y CONTORNOS SOBRE UNA ESFERA (PLTKP, PLTKX, PLTKY)
               [ H.V.M.- MAY-1975        A.B.C.G. - 1992]
*/
#include <math.h>
#include "plotp.h"
#import  <appkit/color.h>
#import  <dpsclient/psops.h>

#define  NT  45
#define  NP  90


double  O1[3][3];                /* Matriz de rotacion de EULER          */
double  EF[NP][NT];              /* Matriz con los valores de la funcion */

void d19sp( PH, TH, P)           /* Funcion de movimiento de la pluma    */
double PH, TH;
int    P;
{
  double the,phi,x,y,z,u,v,w,ro,fi;

   the = 3.14159 * TH;
   phi = 6.28318 * PH;
   x   = sin(the) * cos(phi);
   y   = sin(the) * sin(phi);
   z   = cos(the);
   u   = O1[0][0]*x + O1[0][1]*y + O1[0][2]*z;
   v   = O1[1][0]*x + O1[1][1]*y + O1[1][2]*z;
   w   = O1[2][0]*x + O1[2][1]*y + O1[2][2]*z;
   ro  = sqrt(u*u+v*v);
   fi  = atan2(v,u)/6.28318;
   pltpo(fi, ro, (int)(P && (w >= 0.0)) );
}


void dem19(double e1, double e2, double e3)
{  int     i, j;
   double  s, dt, dp, t, p, tf, pf, ra;
   char    bu[]  = {"Superficie y contornos sobre una esfera"};
   char    bu1[] = {"Funcion de onda tetaedrica"};

   s = -1.0;
   plteu(&O1[0][0], e1, e2, e3);

   dt = 3.14159 / (double)(NT-1);
   dp = 6.28318 / (double)(NP);
   t  = 3.14159;

   for(i = 0 ; i < NT ; i++) {
       p  = 0.475;
       for (j = 0 ; j < NP ; j++) {
	    tf = 0.67 * (1.0 + 0.5*cos(3.0*t) + s*0.1667*cos(9.0*t) );
	    pf = 0.67 * (1.0 + 0.5*sin(3.0*p) - s*0.1667*sin(9.0*p) );
	    ra = tf*pf*0.95;
	    EF[j][i] = ra;
	    p += dp;
       }
       t -= dt;
   }

   pltdk(7);
   plt00();
   plotz("P2 V1 ",6);                   /*  de color negro  el marco */
   pltfr(1);
   plot(2.0,  0.5, UP);                             /*   Marco Superior  */
   plot(-2.0, 0.5, DOWN);      plot(-2.0, 4.5, DOWN);
   plot(2.0,  4.5, DOWN);      plot(2.0,  0.5, DOWN);

   plot(2.0,  -0.5, UP);                            /*   Marco Inferior */
   plot(2.0,  -4.5, DOWN);     plot(-2.0, -4.5, DOWN);
   plot(-2.0, -0.5, DOWN);     plot(2.0,  -0.5, DOWN);

   plttp(-1.4, 0.0, -0.11, bu1, 0.0, (sizeof(bu1) - 1), 1, pltmv);

  pltlla(bu, (sizeof(bu) - 1), bu, 0, 1);
   NXSetColor(NX_COLORBLACK);
   PSstroke();

   pltso(2.0, 2.0, 0.0,  2.5);
   pltsv(&EF[0][0], NP, NT, 1.0, &O1[0][0], vissp, pltpo);
   plotr();
                                                            

   plotz("P1 ",3);                      /*  de color azul la esfera  */
   pltso(1.9, 1.9, 0.0,  -2.5);
   pltkp(0.0 , &EF[0][0], 1.0, 20, 3, NP, 2, NT, d19sp);
   NXSetColor(NX_COLORBLUE);
   PSstroke();

   plotz("P2 ",3);                      /*  rayado en X de negro */
   pltkx(0.50, &EF[0][0], 1.0, NP, NT, d19sp);
   NXSetColor(NX_COLORRED);
   PSstroke();

   plotz("P3 ",3);                      /*  rayado en Y de verde   */
   pltky(0.80, &EF[0][0], 1.0, NP, NT, d19sp);
   NXSetColor(NX_COLORGREEN);
   PSstroke();

   pltig(0.0, 0.0, 1, pltpo);
   pltig(1.0, 1.0, 2, pltpo);
   pltig(0.0, 1.0, 3, pltpo);
   p = 100.0/99.0;
   t = 0.0;
   for(i=0; i<100; i++, t += p) pltig(t, 1.0, 4, pltpo);

   plotr();
                                                            
   pltho();
   pltclose();
}
