#include  <math.h>
#include "../include/ac.h"

/*          ---  FUNCIONES DE ARITMETICA COMPLEJA   ---        
                 [J.P.H.E.           Mayo - 1993]              */



/*                     CREA UN NUMERO COMPLEJO                 */
 
struct complex cmplx(double x, double y)
{
   struct complex t;
   t.x = x;
   t.y = y;
   return t;
}

/*                     VALOR ABSOLUTO DE UN NUMERO COMPLEJO    */

double   acabs(struct complex nc)
{
    return sqrt(nc.x*nc.x + nc.y*nc.y);
}

/*                     ARGUMENTO DE UN NUMERO COMPLEJO        */

double acarc(struct complex z)
{
    double arc;
    arc = 0.0;
    if( (z.x == 0.0) && (z.y == 0.0) ) return arc;
    arc = atan2(z.y, z.x);
    return arc;
}

/*                    RAIZ CUADRADA DE UN NUMERO COMPLEJO   */

struct complex acsqrt(struct complex nc)
{
    double u, v, c;
    c = sqrt(nc.x*nc.x + nc.y*nc.y);
    u = sqrt(0.5*(nc.x + c));
    v = sqrt(0.5*(c - nc.x));
    if( nc.y < 0.0 ) return cmplx(u, -v);
    return cmplx(u,v);
}

/*                   SUMA DE DOS NUMEROS COMPLEJOS       */

struct complex acsum(struct complex c1, struct complex c2)
{
    c1.x += c2.x;
    c1.y += c2.y;
    return c1;
}

/*                   NEGATIVO DE UN NUMERO COMPLEJO        */

struct complex acneg(struct complex nc)
{
    nc.x  = -nc.x;
    nc.y  = -nc.y;
    return nc;
}


/*                 MULTIPLICACION DE DOS NUMEROS COMPLEJOS       */

struct complex acmul(struct complex c1, struct complex c2)
{
    struct complex t;
    t.x = c1.x*c2.x - c1.y*c2.y;
    t.y = c1.x*c2.y + c1.y*c2.x;
    return t;
}

/*                   DIVISION DE DOS NUMEROS COMPLEJOS  C1/C2  */

struct complex acdiv(struct complex c1, struct complex c2)
{
     double  c;
     struct complex t;
     c   = c2.x*c2.x + c2.y*c2.y;
     t.x = (c1.x*c2.x + c1.y*c2.y)/c;
     t.y = (c1.y*c2.x - c1.x*c2.y)/c;
     return t;
}


/*       acsin.c          SENO COMPLEJO                         */

struct complex acsin(struct complex z)
{
   struct complex t, e, i;
   i   =  cmplx(0.0, 2.0);
   t.x = -z.y;
   t.y =  z.x;
   e.x =  z.y;
   e.y = -z.x;
   z   =  acdiv(acsum(acexp(t), acneg(acexp(e))), i);
   return z;
}

/*                       COSENO COMPLEJO                       */

struct complex accos(struct complex z)
{
   struct complex t, e, i;
   i   =  cmplx(2.0, 0.0);
   t.x = -z.y;
   t.y =  z.x;
   e.x =  z.y;
   e.y = -z.x;
   z   =  acdiv(acsum(acexp(t), acexp(e)), i);
   return z;
}

/*                   EXPONENCIAL DE UN NUMERO COMPLEJO       */

struct complex acexp(struct complex nc)
{
    double  e;
    struct complex t;
    e   = exp(nc.x);
    t.x = e * cos(nc.y);
    t.y = e * sin(nc.y);
    return t;
}

/*                   SENO HIPERBOLICO COMPLEJO        */

struct complex acsinh(struct complex z)
{
   struct complex i;
   i   =  cmplx(2.0, 0.0);
   z   =  acdiv( acsum(acexp(z), acneg(acexp(acneg(z)))), i);
   return z;
}

/*                   COSENO HIPERBOLICO COMPLEJO        */

struct complex accosh(struct complex z)
{
   struct complex i;
   i   =  cmplx(2.0, 0.0);
   z   =  acdiv( acsum( acexp(z), acexp(acneg(z))), i);
   return z;
}

/*                    LOGARITMO COMPLEJO                 */
struct complex aclog(struct complex nc)
{
   struct complex r;
   r.x = log(acabs(nc));
   r.y = acarc(nc);
   return r;
}

