#include <math.h>

#include "../include/fontch.h"

#define UP    0
#define DOWN  1

/* CONSTANTS FOR THE FOLLOWING FUNCTION:              */

/* Highest character number in table (starting from 0)*/
#define MAXCH 127 
#define XLAST 8.  /* Ending X point in the character box.               */
#define YLAST 0.  /* Ending Y point in the character box.               */
#define CONTR  94 /* Escape character value for switching to lower-case.*/
#define LASTCH 228/* Starting position of next character, used to
		     terminate an inserted character sequence.
		     The grid coordinates are (6,4) with pen
		     up, giving (6+8) 16+4=228.                         */
#define COFF 64   /* Increment to add to upper-case letter to obtain
		     lower-case in the ascii table.                     */
#define MAXLEN 50 /* Maximum character sequence length allowed. used
		     to prevent loops if a bad table is used.           */
#define UCH  8.

#define xrot(xarg,yarg)  (xarg*c - yarg*s)
#define yrot(xarg,yarg)  (xarg*s + yarg*c)


void plttp(double x, double y, double h, char text[], double theta,
	   int n, int itipo, void (*graff)(double, double, int))

/*

   20-I-89.
   (TEXT STRING PLOTS)
   Esta funcion grafica una cadena de caraacteres en simbolos
   ascii. Los argumentos son:

 (x,y)    Coordenadas del primer caracter, en pulgadas.
  h       Altura de las letras, en pulgadas:
	     Si (h > 0) ==> (X,Y) es el origen del simbolo en
			    la esquina izquierda inferior.
	     Si (h < 0) ==> (X,Y) es el centro del simbolo.
 texto[]  Arreglo donde se encuentran los caracteres del texto,
	  uno por byte. (logico).
 theta    Angulo de rotacion de los caracteres respecto al eje X,
	  en grados.
 n        Numero de caracteres del texto.
 itipo    Tipo de carateres.
 graff    Funcion de conversion de corrdenadas.


   The characters are built up of points in a box 15 units high and
   8 wide.  The lower left corner of a symbol is at (0,4), allowing
   certain lower case letters (g,j,q,y) to hang below the line.
   The center of a symbol is at (3,9). Capital letters and digits
   are 10 units high and 6 wide, and 2 extra units in the width are
   used to ensure spacing between adjacent symbols. The actual
   width of a symbol is thus 0.6 abs(h), and the actual length
   of a text string will be 0.8 abs(h) float(iabs(n)). Since
   ascii characters 0-31 have no printer graphic, a set of
   spacial symbols useful in marking plot data has bess
   implemented.

   The coding here has been arranged very carefully to be ANSI
   Fortran and word length independent. The character set of any
   particular machine may be defined by altering a single pointer
   array below. The machine-dependent unpacking of characters
   from a word is localized in subroutine getch which must br
   changed for each machine.

   In this version, hollerith text in text[] is assumed to be
   in the CDC 64-character set, modified to allow 96 character
   ascii by adding lower-case letters.
   This is done by dedicating a character to an escape character
   (set in a data statement below to a semicolon) which indicates
   that the next character following is lower-case alphabetic.
   The escape character is not plotted, but must be included in
   the character count in the argument list. 
   In order to plot the escape character, it must be coded as
   two sucessive characters in text[].

   (30-SEP-77) (Written by Nelson H.F. Beebe, Aarhus, Denmark)
   (14-ENE-85) Modificaciones (PLOT84 Written by Francisco Gil).
   (29-JUN-88) Conversion a C, G.Cisneros.
   (21-Ene-89) Angel Canto.
*/
{
int       up, *indx, letra, length, ix, iy;
char      *coor, *coopt;
double    c, s, delta, xend, yend, dx, dy;

 if (n == 0 || h == 0.0)
 return;
 switch(itipo)
 {
  case 2:  coor = bfxy;
	   indx = lbfc;
	   break;
  case 1:
  default:
	   coor = asxy;
	   indx = lasc;
	   break;
 }

 c  = cos(theta * 1.7453293E-02);
 s  = sin(theta * 1.7453293E-02);
 delta = fabs(h) / UCH;
 if (h < 0.0)
 {  x -= xrot(3.0 * delta, 5.0 * delta);
    y -= yrot(3.0 * delta, 5.0 * delta);
 }
 xend = XLAST * delta;
 yend = YLAST * delta;
 while (n--)
 {  letra = *text++;
    letra = (letra < 0) ? 0 : ((letra > MAXCH) ? MAXCH :letra);
    if (letra == CONTR)
    {
       if ((n--) <= 0) return;
       letra = *text++;
       letra = (letra < 0) ? 0 : ((letra > MAXCH) ? MAXCH : letra);
       if ((letra != CONTR) && (letra >= COFF))  letra -= COFF;
    }
    length = indx[letra+1] - indx[letra];
    length = (length > MAXLEN || length <= 0) ? MAXLEN : length;
    coopt = coor + indx[letra];
    while (length--)
    {  if (*coopt == LASTCH) break;
       ix = *coopt >> 4;
       iy = (*coopt++ & 0xf) - 4;
       up = ix & 0x8;
       ix &= 0x7;
       dx = (double) ix * delta;
       dy = (double) iy * delta;
       (*graff)(x + xrot(dx, dy), y + yrot(dx, dy), !up);
    }
    x += xrot(xend, yend);
    y += yrot(xend, yend);
    (*graff)(x, y, UP);
 }
}

/* END: plttp.c */

