#include "vis.h"

#define Q0 0.000
#define Q1 1.571
#define Q2 3.142
#define Q3 4.713
#define Q4 6.283
#define S1 1.570
#define S3 4.712
#define X1 0.01
#define X2 1.76

void pltev(double Z1, double ZE[], double Z2, int NX, 
           int NE, void (*pl)())

/*       pltev.c
   [Eliptical view].
   Vista eliptica.
   Programa para producir un dibujo en perspectiva de una funcion e-
   valuada y definida sobre coordenadas elipticas, para exhibir
   los arcos corespondientes a las constantes XI y ETA.
   X = COSH(XI)*COS(ETA).
   Y = SINH(XI)*SIN(ETA).
   ZE[NX][NE].

   (Z1,Z2)   Rango de los valores de ZE.
   ZE[]      Arreglo con los valores de la funcion.
   NX        Numero de valores XI.
   NE        Numero (=4*N+1) de valores ETA.
   pl        Funcion de conversion de coordenadas.
   (H.V.McIntosh, 17.MAY.75)
   (Fco. Gil, 30.DIC.84)
   (Conv. a C, G. Cisneros, 15.JUL.88)
   (Angel Canto, 16.Mar.89)
*/
{
  int nq,nn;

 nq = 1 + (NE-1)/4;
 nn = nq - 1;
 visnh();
 vises(Z1, &ZE[3*nn], Z2, X1, X2, NX, Q3, Q4, nq, -1,  1, pl);
 vises(Z1, &ZE[0   ], Z2, X1, X2, NX, Q0, S1, nq,  1,  1, pl);
 visnh();
 vises(Z1, &ZE[2*nn], Z2, X1, X2, NX, Q2, S3, nq, -1, -1, pl);
 vises(Z1, &ZE[nn  ], Z2, X1, X2, NX, Q1, Q2, nq,  1, -1, pl);
}

