#include "vis.h"

void pltsv(double FU[], int NP, int NT, double S, double O[],
	   void (*pr)(), void (*pl)())

/*       pltsv.c
   [Spherical view]
   Vista esferica.
   Programa para prodicir un dibujo en perspectiva de una funcion e-
   valuada y definida sobre una superficie esferica, para exhibir  -
   los arcos de latitud y longitud.
   FU[]   Arreglo de los valores de la funcion (FU[NP][NT])
   NP     Numero (=2*N) de puntos en una latitud.
   NT     Numero de puntos en una longitud.
   S       S =  1, Grafica de FU positiva.
	   S = -1, Grafica de FU negativa.
   O[]    Matriz ortogonal de rotacion.
   pr     Funcion de proyeccion.
   pl     Funcion de conversion de coordenadas.
   (H.V.McIntosh, 15.MAY.75)
   (Fco. Gil, 23.DIC.84)
   (Conv. a C, G. Cisneros, 15.JUL.88)
   (Angel Canto, 16.Mar.89)
*/
{
  int    b, c, nh, i1, i2, i3, i4, j1, j2, j3, j4, jp, it;
  double s1, s2, ph, th, r, p;

 nh = NP / 2;
 visnp(&ph, &th, &jp, &it, NP, NT, O);

 if (th <= (1.57079))
 { i1 = 1;
   i2 = it;
   i3 = it;
   i4 = NT;
   s1 = 1.0;
   s2 = -1.0; }
 else
   {    i1 = it;
	i2 = NT;
	i3 = 1;
	i4 = it;
	s1 = -1.0;
	s2 = 1.0; };

 j1 = jp;
 j2 = jp+nh;
 j3 = jp-nh;
 j4 = jp;
 (*pr)(&r, &p, 0.1, th, ph+0.05, O);
 b = ((-0.25) < p) && (p <= (0.25));
 c = !b;

 visnh();
 visss(FU, j1, j2, NP, i1, i2, NT, 1,-1, s1, b, S, O, pr, pl);
 visss(FU, j1, j2, NP, i1, i2, NT, 1, 1, s2, b, S, O, pr, pl);
 visss(FU, j1, j2, NP, i3, i4, NT, 1, 1, s2, b, S, O, pr, pl);
 visss(FU, j1, j2, NP, i3, i4, NT, 1,-1, s1, b, S, O, pr, pl);
 visnh();
 visss(FU, j3, j4, NP, i1, i2, NT,-1,-1, s1, c, S, O, pr, pl);
 visss(FU, j3, j4, NP, i1, i2, NT,-1, 1, s2, c, S, O, pr, pl);
 visss(FU, j3, j4, NP, i3, i4, NT,-1, 1, s2, c, S, O, pr, pl);
 visss(FU, j3, j4, NP, i3, i4, NT,-1,-1, s1, c, S, O, pr, pl);
}

/* END: pltsv */

