#include "vis.h"

#define SIZE 150

void pvvds(double Z1, double ZE[], double Z2, int J1, int J2, 
	   int MX, int I1, int I2, int MY, double US, double VS, 
	   int L, int M, double S, void (*pl)())

/*       pvvds.c
   [Diagonal sequence]
   (Fco. Gil, 29.DIC.84)
   (Conv. a C, G. Cisneros, 15.JUL.88)
   (Angel Canto, 16.MAR.89)
   Secuencia diagonal.
   Esta funcion es equivalante a "pvids", salvo que esta grafica
   los valores positivos y negativos reales. En el caso de "pvids"
   despues de checar el signo, toma el valor absoluto de la funcion
   para graficar.

   (Z1,Z2)   Rango de los valores de ZE.
   J1,J2     Rango de X (coordenada horizontal).
   MX,MY     ZE[MX][MY].
   I1,I2     Rango de Y (Coordenada de profundidad).
   US,VS     Cortes totales en direcc. U y V (Max. 1.0).
   L         Direcc. de vista e incremento (+:Oeste, -:Este).
   M         Direcc. de vista e incremento (+:Sur, -:Norte).
   S         =1.0 Grafica parte positiva, = -1.0 Parte negativa.
   pl        Funcion de conversion de coordenadas. (usual pltca)
*/
{
  int P[SIZE];
  double  U[SIZE],V[SIZE];
  int nl,nm,n,il,iu,mm,nn,i0,j0,k0,i,j,k,ix,kk;
  double te,zs,dui,eui,duj,euj,dvi,evi,eu,ve;

 nl = sgn(L);
 nm = sgn(M);
 n = nl * nm;
 il = I1 - abs(M);
 iu = I2 + abs(M);
 mm = L * MY;
 nn = 1;
 te = 0.5*(double)(nl+1);
 zs = (1.0-VS) / (Z2-Z1);
 dui = -((double)(nl)*US) / (double)(MY-1);
 eui = dui * (double)(M);
 duj = (1.0-US) / (double)(MX-1);
 euj = duj * (double)(L);
 dvi = VS / (double)(MY-1);
 evi = dvi * (double)(M);

 i0 = (nm > 0) ? (I1+M) : (I2+M);
 j0 = (nl > 0) ? J1 : J2;
 k0 = (n > 0) ? -1 : SIZE;

 do{
    do{
	k = k0;
	i = visbd(i0, iu, il);
	j = j0;
	ix = (i-1) + (j-1)*MY;
	eu = te*US + dui*(double)(i-1) + duj*(double)(j-1);
	ve = dvi*(double)(i-1);

	do{
	   if ((i >= I1) && (i <= I2))
	    {
	      k = visbd(k+n, SIZE-1, 0);
	      U[k] = (double)(eu);
	      V[k] = (double)(ve + zs*(ZE[ix]-Z1));
	      P[k] = (int)((S*ZE[ix] >= 0.0));  };

	   i -= M;
	   ix -= M;
	   eu -= eui;
	   ve -= evi;
	   if ((i < I1) || (i > I2)) break;
	   U[k] = (double)(eu);
	   V[k] = (double)(ve+zs*(ZE[ix]-Z1));
	   P[k] = (int)((S*ZE[ix] >= 0.0));
	   j += L;
	   ix += mm;
	   eu += euj;
	  } while ((j >= J1) && (j <= I2));

	kk = ((k+2-n*k)/2) -1;
	visch (&U[kk], &V[kk], &P[kk], 
	       (int)((SIZE+1-n*(SIZE-2*(k+1)+1))/2), nn, pl);
	nn = -nn;
	i0 += M;
      } while ((i0 >=  il) && (i0 <= iu));
  j0 += L;
   } while ((j0 >= J1) && (j0 <= J2));
}
