 #include "vis.h"

#include <math.h>

#define SIZE 550

static  double  U[SIZE],V[SIZE];

void vises(double Z1, double ZE[], double Z2, double X1, double X2,
	   int NX, double E1, double E2, int NE, int L, int M,
	   void (*pl)())

/*       vises.c
   [Elliptical sequence]
   Secuencia eliptica.
   Dimension total ZE[NX][4*(NE-1)+1].

   (Z1,Z2)   Rango de los valores de "ZE".
   ZE[]      Arreglo con los valores ZE[NX][NE].
   X1,X2     Rango de XI.
   NX        Numero de valores XI.
   E1,E2     Rango de ETA.
   NE        Numero de valores de ETA.
   L          =1 Oeste , =-1 Este.
   M          =1 Sur   , =-1 Norte.
   pl        Funcion de conversion de coordenadas.
   (H.V.McIntosh, 17.MAY.75)
   (Fco. Gil, 30.DIC.84)
   (Conv. a C, G. Cisneros, 8.JUL.88)
   (Angel Canto, 16 Mar.89)
*/
{
  int n,mm,i0,j0,i,j,k,ixji,k0,net;
  double el,em,dx,de,zs,e,x;


 net = 4*(NE-1) + 1;
 n   = L * M;
 mm  = 1;
 el  = (double)(L);
 em  = (double)(M);
 dx  = (X2-X1) / (double)(NX-1);
 de  = (E2-E1) / (double)(NE-1);
 zs  = 0.58 / (Z2-Z1);

 i0  = (M > 0) ? 2 : NE-1;
 j0  = (L > 0) ? 1 : NX;
 k0  = (n > 0) ? -1 : SIZE;

 do{
    do{
	i = visbd(i0, NE+1, 0);
	j = j0;
	k = k0;
	e = E1 + de*(double)(i-1);
	x = X1 + dx*(double)(j-1);

	do{
	    if((i >= 1) && (i <= NE))
	     {
	       k = visbd(k+n, SIZE-1, 0);
	       U[k] = (double)(0.166 * (cosh(x) * cos(e) + 3.0));
	       ixji = (i-1) + (net * (j-1));
	       V[k] = (double)((zs*(ZE[ixji]-Z1) + 0.2) + 
		       0.075 * sinh(x) * sin(e));
	     };
	    i -= M;
	    e -= em * de;
	    if ((i < 1) || (i > NE)) break;
	    k = visbd(k+n, SIZE-1, 0);
	    U[k] = (double)(0.166 * (cosh(x) * cos(e) + 3.0));
	    ixji = (i-1) +  (net*(j-1));
	    V[k] = (double)((zs*(ZE[ixji]-Z1) + 0.2) + 
		   0.075 * sinh(x) * sin(e));
	    j += L;
	    x += el * dx;
	  } while ((j >= 1) && (j <= NX));

	if(n > 0) visho(U, V, k+1, mm, pl);

	if(n < 0) visho(&U[k], &V[k], SIZE-k, mm, pl);
	mm = -mm;
	i0 += M;
      } while ((i0 >= 0) && (i0 <= NE+1));
	j0 += L;
   } while ((j0 >= 1) && (j0 <= NX));
}

/* END: vises.c */

