#include <math.h>

#include "vis.h"

void visnp(double *PH, double *TH, int *JP, int *IT, int NP, int NT,
	   double O[])

/*       visnp.c
   [Nearest point]
   Punto cercano.
   Determina las coordenadas de la linea, observando desde el centro
   de la esfera.

   *PH,*TH  Coordenadas de la superficie del punto cercano.
   *JP,*IT  Indice del punto cercano.
   NP,NT    Rango de los puntos de PHI, THETA.
   O[]      Matriz ortogonal (matriz de rotacion)
   (H.V.McIntosh, 16.JUN.74)
   (Fco. Gil, 23.DIC.84)
   (Conv. a C, G. Cisneros, 8.JUL.88)
   (Angel Canto, 16 Mar.89)
*/
{
  double o20,o21,rh;

  o20 = O[6];
  o21 = O[7];
  *PH = atan2(o21,o20);
  if(*PH < 0.0) *PH += 6.28318;
  rh = sqrt(o20*o20+o21*o21);
  *TH = atan2(rh,O[8]);
  *JP = 1+(int)(0.5+((double)(NP)* (*PH))/6.28318);
  *IT = visbd( (int)(1.5+((*TH)*(double)(NT-1))/3.14159), NT, 1);
}
