#include <math.h>
#include "vis.h"

#define SIZE 600

static  double  U[SIZE],V[SIZE];

void visps(double Z1, double ZE[], double Z2, double R1, 
	   double R2, int NR, double P1, double P2, int NP, 
	   int L, int M, void (*pl)())

/*       visps.c
   [Polar sequence]
   Secuencia Polar.
   Dimension total ZE[NR][4*(NP-1)+1].

   (Z1,Z2)  Rango de los valores de la funcion.
   ZE[]     Arreglo de valores ZE[NR][NP].
   NR       Numero de puntos en R.
   R1,R2    Intervalo en R.
   NP       Numero de puntos en P.
   P1,P2    Intervalo en P.
   L        L = (1= Oeste, -1= Este).
   M        M = (1= Sur  , -1= Norte).
   pl       Funcion de conversion de coordenadas.
   (H.V.McIntosh, 17.MAY.75)
   (Fco. Gil, 26.DIC.84)
   (Conv. a C, G. Cisneros, 8.JUL.88)
   (Angel Canto, 16 Mar.89)
*/
 {
  int   n,mm,i0,j0,k0,i,j,k,ixji,npt;
  double el,em,zs,dp,dr,p,r;

  npt = 4 * (NP-1)+1;
  n = L * M;
  mm = 1;
  el = (double)(L);
  em = (double)(M);
  zs = 0.66667 / (Z2-Z1);
  dp = (P2-P1) / (double)(NP-1);
  dr = (R2-R1) / (double)(NR-1);

  i0 = (M > 0) ? 2 : (NP-1);
  j0 = (L > 0) ? 1 : NR;
  k0 = (n > 0) ? -1 : SIZE ;

 do{
    do{
	k = k0;
	i = visbd(i0, NP+1, 0);
	j = j0;
	p = P1 + dp * (double)(i-1);
	r = R1 + dr * (double)(j-1);

	do{
	    if((i <= NP) && (i >= 1))
	    {
	     k = visbd(k+n, SIZE-1, 0);
	     U[k] = (double)(0.5*(1.0 + r * cos(p)));
	     ixji = (i-1) + (npt * (j-1));
     V[k] = (double)((0.166667 + zs*(ZE[ixji]-Z1)) + 
	     0.166667* r *sin(p) ); };
	     i -= M;
	     p -= em * dp;
	     if((i > NP) || (i < 1)) break;
	     k = visbd(k+n, SIZE-1, 0);
	     U[k] = (double)(0.5*(1.0 + r*cos(p)));
	     ixji = (i-1) + (npt * (j-1));
     V[k] = (double)((0.166667 + zs*(ZE[ixji]-Z1)) + 0.166667* r *sin(p)) ;
	     j += L;
	     r += el * dr;
	    } while ((j <= NR) && (j >= 1));

	if (n > 0) visho(U, V, k+1, mm, pl);
	if (n < 0) visho(&U[k], &V[k], SIZE-k, mm, pl);
	mm = -mm;
	i0 += M;
      } while ((i0 >= 0) && (i0 <= NP+1));
	j0 += L;
   } while ((j0 >= 1) && (j0 <= NR));
}

/* END: visps.c */

